"""
Pydantic Models for CMDB - firewall/profile_group

Runtime validation models for firewall/profile_group configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class ProfileGroupModel(BaseModel):
    """
    Pydantic model for firewall/profile_group configuration.
    
    Configure profile groups.
    
    Validation Rules:        - name: max_length=47 pattern=        - profile_protocol_options: max_length=47 pattern=        - ssl_ssh_profile: max_length=47 pattern=        - av_profile: max_length=47 pattern=        - webfilter_profile: max_length=47 pattern=        - dnsfilter_profile: max_length=47 pattern=        - emailfilter_profile: max_length=47 pattern=        - dlp_profile: max_length=47 pattern=        - file_filter_profile: max_length=47 pattern=        - ips_sensor: max_length=47 pattern=        - application_list: max_length=47 pattern=        - voip_profile: max_length=47 pattern=        - ips_voip_filter: max_length=47 pattern=        - sctp_filter_profile: max_length=47 pattern=        - diameter_filter_profile: max_length=47 pattern=        - virtual_patch_profile: max_length=47 pattern=        - icap_profile: max_length=47 pattern=        - videofilter_profile: max_length=47 pattern=        - waf_profile: max_length=47 pattern=        - ssh_filter_profile: max_length=47 pattern=        - casb_profile: max_length=47 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=47, description="Profile group name.")    
    profile_protocol_options: str | None = Field(max_length=47, default="default", description="Name of an existing Protocol options profile.")  # datasource: ['firewall.profile-protocol-options.name']    
    ssl_ssh_profile: str | None = Field(max_length=47, default="certificate-inspection", description="Name of an existing SSL SSH profile.")  # datasource: ['firewall.ssl-ssh-profile.name']    
    av_profile: str | None = Field(max_length=47, default=None, description="Name of an existing Antivirus profile.")  # datasource: ['antivirus.profile.name']    
    webfilter_profile: str | None = Field(max_length=47, default=None, description="Name of an existing Web filter profile.")  # datasource: ['webfilter.profile.name']    
    dnsfilter_profile: str | None = Field(max_length=47, default=None, description="Name of an existing DNS filter profile.")  # datasource: ['dnsfilter.profile.name']    
    emailfilter_profile: str | None = Field(max_length=47, default=None, description="Name of an existing email filter profile.")  # datasource: ['emailfilter.profile.name']    
    dlp_profile: str | None = Field(max_length=47, default=None, description="Name of an existing DLP profile.")  # datasource: ['dlp.profile.name']    
    file_filter_profile: str | None = Field(max_length=47, default=None, description="Name of an existing file-filter profile.")  # datasource: ['file-filter.profile.name']    
    ips_sensor: str | None = Field(max_length=47, default=None, description="Name of an existing IPS sensor.")  # datasource: ['ips.sensor.name']    
    application_list: str | None = Field(max_length=47, default=None, description="Name of an existing Application list.")  # datasource: ['application.list.name']    
    voip_profile: str | None = Field(max_length=47, default=None, description="Name of an existing VoIP (voipd) profile.")  # datasource: ['voip.profile.name']    
    ips_voip_filter: str | None = Field(max_length=47, default=None, description="Name of an existing VoIP (ips) profile.")  # datasource: ['voip.profile.name']    
    sctp_filter_profile: str | None = Field(max_length=47, default=None, description="Name of an existing SCTP filter profile.")  # datasource: ['sctp-filter.profile.name']    
    diameter_filter_profile: str | None = Field(max_length=47, default=None, description="Name of an existing Diameter filter profile.")  # datasource: ['diameter-filter.profile.name']    
    virtual_patch_profile: str | None = Field(max_length=47, default=None, description="Name of an existing virtual-patch profile.")  # datasource: ['virtual-patch.profile.name']    
    icap_profile: str | None = Field(max_length=47, default=None, description="Name of an existing ICAP profile.")  # datasource: ['icap.profile.name']    
    videofilter_profile: str | None = Field(max_length=47, default=None, description="Name of an existing VideoFilter profile.")  # datasource: ['videofilter.profile.name']    
    waf_profile: str | None = Field(max_length=47, default=None, description="Name of an existing Web application firewall profile.")  # datasource: ['waf.profile.name']    
    ssh_filter_profile: str | None = Field(max_length=47, default=None, description="Name of an existing SSH filter profile.")  # datasource: ['ssh-filter.profile.name']    
    casb_profile: str | None = Field(max_length=47, default=None, description="Name of an existing CASB profile.")  # datasource: ['casb.profile.name']    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('profile_protocol_options')
    @classmethod
    def validate_profile_protocol_options(cls, v: Any) -> Any:
        """
        Validate profile_protocol_options field.
        
        Datasource: ['firewall.profile-protocol-options.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('ssl_ssh_profile')
    @classmethod
    def validate_ssl_ssh_profile(cls, v: Any) -> Any:
        """
        Validate ssl_ssh_profile field.
        
        Datasource: ['firewall.ssl-ssh-profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('av_profile')
    @classmethod
    def validate_av_profile(cls, v: Any) -> Any:
        """
        Validate av_profile field.
        
        Datasource: ['antivirus.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('webfilter_profile')
    @classmethod
    def validate_webfilter_profile(cls, v: Any) -> Any:
        """
        Validate webfilter_profile field.
        
        Datasource: ['webfilter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('dnsfilter_profile')
    @classmethod
    def validate_dnsfilter_profile(cls, v: Any) -> Any:
        """
        Validate dnsfilter_profile field.
        
        Datasource: ['dnsfilter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('emailfilter_profile')
    @classmethod
    def validate_emailfilter_profile(cls, v: Any) -> Any:
        """
        Validate emailfilter_profile field.
        
        Datasource: ['emailfilter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('dlp_profile')
    @classmethod
    def validate_dlp_profile(cls, v: Any) -> Any:
        """
        Validate dlp_profile field.
        
        Datasource: ['dlp.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('file_filter_profile')
    @classmethod
    def validate_file_filter_profile(cls, v: Any) -> Any:
        """
        Validate file_filter_profile field.
        
        Datasource: ['file-filter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('ips_sensor')
    @classmethod
    def validate_ips_sensor(cls, v: Any) -> Any:
        """
        Validate ips_sensor field.
        
        Datasource: ['ips.sensor.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('application_list')
    @classmethod
    def validate_application_list(cls, v: Any) -> Any:
        """
        Validate application_list field.
        
        Datasource: ['application.list.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('voip_profile')
    @classmethod
    def validate_voip_profile(cls, v: Any) -> Any:
        """
        Validate voip_profile field.
        
        Datasource: ['voip.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('ips_voip_filter')
    @classmethod
    def validate_ips_voip_filter(cls, v: Any) -> Any:
        """
        Validate ips_voip_filter field.
        
        Datasource: ['voip.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('sctp_filter_profile')
    @classmethod
    def validate_sctp_filter_profile(cls, v: Any) -> Any:
        """
        Validate sctp_filter_profile field.
        
        Datasource: ['sctp-filter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('diameter_filter_profile')
    @classmethod
    def validate_diameter_filter_profile(cls, v: Any) -> Any:
        """
        Validate diameter_filter_profile field.
        
        Datasource: ['diameter-filter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('virtual_patch_profile')
    @classmethod
    def validate_virtual_patch_profile(cls, v: Any) -> Any:
        """
        Validate virtual_patch_profile field.
        
        Datasource: ['virtual-patch.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('icap_profile')
    @classmethod
    def validate_icap_profile(cls, v: Any) -> Any:
        """
        Validate icap_profile field.
        
        Datasource: ['icap.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('videofilter_profile')
    @classmethod
    def validate_videofilter_profile(cls, v: Any) -> Any:
        """
        Validate videofilter_profile field.
        
        Datasource: ['videofilter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('waf_profile')
    @classmethod
    def validate_waf_profile(cls, v: Any) -> Any:
        """
        Validate waf_profile field.
        
        Datasource: ['waf.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('ssh_filter_profile')
    @classmethod
    def validate_ssh_filter_profile(cls, v: Any) -> Any:
        """
        Validate ssh_filter_profile field.
        
        Datasource: ['ssh-filter.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('casb_profile')
    @classmethod
    def validate_casb_profile(cls, v: Any) -> Any:
        """
        Validate casb_profile field.
        
        Datasource: ['casb.profile.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ProfileGroupModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_profile_protocol_options_references(self, client: Any) -> list[str]:
        """
        Validate profile_protocol_options references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/profile-protocol-options        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     profile_protocol_options="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_profile_protocol_options_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "profile_protocol_options", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.profile_protocol_options.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Profile-Protocol-Options '{value}' not found in "
                "firewall/profile-protocol-options"
            )        
        return errors    
    async def validate_ssl_ssh_profile_references(self, client: Any) -> list[str]:
        """
        Validate ssl_ssh_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/ssl-ssh-profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     ssl_ssh_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ssl_ssh_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ssl_ssh_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.ssl_ssh_profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ssl-Ssh-Profile '{value}' not found in "
                "firewall/ssl-ssh-profile"
            )        
        return errors    
    async def validate_av_profile_references(self, client: Any) -> list[str]:
        """
        Validate av_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - antivirus/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     av_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_av_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "av_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.antivirus.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Av-Profile '{value}' not found in "
                "antivirus/profile"
            )        
        return errors    
    async def validate_webfilter_profile_references(self, client: Any) -> list[str]:
        """
        Validate webfilter_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - webfilter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     webfilter_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_webfilter_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "webfilter_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.webfilter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Webfilter-Profile '{value}' not found in "
                "webfilter/profile"
            )        
        return errors    
    async def validate_dnsfilter_profile_references(self, client: Any) -> list[str]:
        """
        Validate dnsfilter_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - dnsfilter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     dnsfilter_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_dnsfilter_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "dnsfilter_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.dnsfilter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Dnsfilter-Profile '{value}' not found in "
                "dnsfilter/profile"
            )        
        return errors    
    async def validate_emailfilter_profile_references(self, client: Any) -> list[str]:
        """
        Validate emailfilter_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - emailfilter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     emailfilter_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_emailfilter_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "emailfilter_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.emailfilter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Emailfilter-Profile '{value}' not found in "
                "emailfilter/profile"
            )        
        return errors    
    async def validate_dlp_profile_references(self, client: Any) -> list[str]:
        """
        Validate dlp_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - dlp/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     dlp_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_dlp_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "dlp_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.dlp.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Dlp-Profile '{value}' not found in "
                "dlp/profile"
            )        
        return errors    
    async def validate_file_filter_profile_references(self, client: Any) -> list[str]:
        """
        Validate file_filter_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - file-filter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     file_filter_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_file_filter_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "file_filter_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.file_filter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"File-Filter-Profile '{value}' not found in "
                "file-filter/profile"
            )        
        return errors    
    async def validate_ips_sensor_references(self, client: Any) -> list[str]:
        """
        Validate ips_sensor references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - ips/sensor        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     ips_sensor="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ips_sensor_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ips_sensor", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.ips.sensor.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ips-Sensor '{value}' not found in "
                "ips/sensor"
            )        
        return errors    
    async def validate_application_list_references(self, client: Any) -> list[str]:
        """
        Validate application_list references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - application/list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     application_list="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_application_list_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "application_list", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.application.list.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Application-List '{value}' not found in "
                "application/list"
            )        
        return errors    
    async def validate_voip_profile_references(self, client: Any) -> list[str]:
        """
        Validate voip_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - voip/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     voip_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_voip_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "voip_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.voip.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Voip-Profile '{value}' not found in "
                "voip/profile"
            )        
        return errors    
    async def validate_ips_voip_filter_references(self, client: Any) -> list[str]:
        """
        Validate ips_voip_filter references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - voip/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     ips_voip_filter="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ips_voip_filter_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ips_voip_filter", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.voip.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ips-Voip-Filter '{value}' not found in "
                "voip/profile"
            )        
        return errors    
    async def validate_sctp_filter_profile_references(self, client: Any) -> list[str]:
        """
        Validate sctp_filter_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - sctp-filter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     sctp_filter_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_sctp_filter_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "sctp_filter_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.sctp_filter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Sctp-Filter-Profile '{value}' not found in "
                "sctp-filter/profile"
            )        
        return errors    
    async def validate_diameter_filter_profile_references(self, client: Any) -> list[str]:
        """
        Validate diameter_filter_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - diameter-filter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     diameter_filter_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_diameter_filter_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "diameter_filter_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.diameter_filter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Diameter-Filter-Profile '{value}' not found in "
                "diameter-filter/profile"
            )        
        return errors    
    async def validate_virtual_patch_profile_references(self, client: Any) -> list[str]:
        """
        Validate virtual_patch_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - virtual-patch/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     virtual_patch_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_virtual_patch_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "virtual_patch_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.virtual_patch.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Virtual-Patch-Profile '{value}' not found in "
                "virtual-patch/profile"
            )        
        return errors    
    async def validate_icap_profile_references(self, client: Any) -> list[str]:
        """
        Validate icap_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - icap/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     icap_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_icap_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "icap_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.icap.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Icap-Profile '{value}' not found in "
                "icap/profile"
            )        
        return errors    
    async def validate_videofilter_profile_references(self, client: Any) -> list[str]:
        """
        Validate videofilter_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - videofilter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     videofilter_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_videofilter_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "videofilter_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.videofilter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Videofilter-Profile '{value}' not found in "
                "videofilter/profile"
            )        
        return errors    
    async def validate_waf_profile_references(self, client: Any) -> list[str]:
        """
        Validate waf_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - waf/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     waf_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_waf_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "waf_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.waf.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Waf-Profile '{value}' not found in "
                "waf/profile"
            )        
        return errors    
    async def validate_ssh_filter_profile_references(self, client: Any) -> list[str]:
        """
        Validate ssh_filter_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - ssh-filter/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     ssh_filter_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ssh_filter_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ssh_filter_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.ssh_filter.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ssh-Filter-Profile '{value}' not found in "
                "ssh-filter/profile"
            )        
        return errors    
    async def validate_casb_profile_references(self, client: Any) -> list[str]:
        """
        Validate casb_profile references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - casb/profile        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileGroupModel(
            ...     casb_profile="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_casb_profile_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.profile_group.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "casb_profile", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.casb.profile.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Casb-Profile '{value}' not found in "
                "casb/profile"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_profile_protocol_options_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ssl_ssh_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_av_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_webfilter_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_dnsfilter_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_emailfilter_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_dlp_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_file_filter_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ips_sensor_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_application_list_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_voip_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ips_voip_filter_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_sctp_filter_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_diameter_filter_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_virtual_patch_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_icap_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_videofilter_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_waf_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_ssh_filter_profile_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_casb_profile_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ProfileGroupModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.532108Z
# ============================================================================


"""
Pydantic Models for CMDB - firewall/shaper/traffic_shaper

Runtime validation models for firewall/shaper/traffic_shaper configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class TrafficShaperModel(BaseModel):
    """
    Pydantic model for firewall/shaper/traffic_shaper configuration.
    
    Configure shared traffic shaper.
    
    Validation Rules:        - name: max_length=35 pattern=        - guaranteed_bandwidth: min=0 max=80000000 pattern=        - maximum_bandwidth: min=0 max=80000000 pattern=        - bandwidth_unit: pattern=        - priority: pattern=        - per_policy: pattern=        - diffserv: pattern=        - diffservcode: pattern=        - dscp_marking_method: pattern=        - exceed_bandwidth: min=0 max=80000000 pattern=        - exceed_dscp: pattern=        - maximum_dscp: pattern=        - cos_marking: pattern=        - cos_marking_method: pattern=        - cos: pattern=        - exceed_cos: pattern=        - maximum_cos: pattern=        - overhead: min=0 max=100 pattern=        - exceed_class_id: min=0 max=4294967295 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=35, default=None, description="Traffic shaper name.")    
    guaranteed_bandwidth: int | None = Field(ge=0, le=80000000, default=0, description="Amount of bandwidth guaranteed for this shaper (0 - 80000000). Units depend on the bandwidth-unit setting.")    
    maximum_bandwidth: int | None = Field(ge=0, le=80000000, default=0, description="Upper bandwidth limit enforced by this shaper (0 - 80000000). 0 means no limit. Units depend on the bandwidth-unit setting.")    
    bandwidth_unit: Literal["kbps", "mbps", "gbps"] | None = Field(default="kbps", description="Unit of measurement for guaranteed and maximum bandwidth for this shaper (Kbps, Mbps or Gbps).")    
    priority: Literal["low", "medium", "high"] | None = Field(default="high", description="Higher priority traffic is more likely to be forwarded without delays and without compromising the guaranteed bandwidth.")    
    per_policy: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable applying a separate shaper for each policy. For example, if enabled the guaranteed bandwidth is applied separately for each policy.")    
    diffserv: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable changing the DiffServ setting applied to traffic accepted by this shaper.")    
    diffservcode: str | None = Field(default=None, description="DiffServ setting to be applied to traffic accepted by this shaper.")    
    dscp_marking_method: Literal["multi-stage", "static"] | None = Field(default="static", description="Select DSCP marking method.")    
    exceed_bandwidth: int | None = Field(ge=0, le=80000000, default=0, description="Exceed bandwidth used for DSCP/VLAN CoS multi-stage marking. Units depend on the bandwidth-unit setting.")    
    exceed_dscp: str | None = Field(default=None, description="DSCP mark for traffic in guaranteed-bandwidth and exceed-bandwidth.")    
    maximum_dscp: str | None = Field(default=None, description="DSCP mark for traffic in exceed-bandwidth and maximum-bandwidth.")    
    cos_marking: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable VLAN CoS marking.")    
    cos_marking_method: Literal["multi-stage", "static"] | None = Field(default="static", description="Select VLAN CoS marking method.")    
    cos: str | None = Field(default=None, description="VLAN CoS mark.")    
    exceed_cos: str | None = Field(default=None, description="VLAN CoS mark for traffic in [guaranteed-bandwidth, exceed-bandwidth].")    
    maximum_cos: str | None = Field(default=None, description="VLAN CoS mark for traffic in [exceed-bandwidth, maximum-bandwidth].")    
    overhead: int | None = Field(ge=0, le=100, default=0, description="Per-packet size overhead used in rate computations.")    
    exceed_class_id: int | None = Field(ge=0, le=4294967295, default=0, description="Class ID for traffic in guaranteed-bandwidth and maximum-bandwidth.")  # datasource: ['firewall.traffic-class.class-id']    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('exceed_class_id')
    @classmethod
    def validate_exceed_class_id(cls, v: Any) -> Any:
        """
        Validate exceed_class_id field.
        
        Datasource: ['firewall.traffic-class.class-id']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "TrafficShaperModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_exceed_class_id_references(self, client: Any) -> list[str]:
        """
        Validate exceed_class_id references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - firewall/traffic-class        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = TrafficShaperModel(
            ...     exceed_class_id="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_exceed_class_id_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.firewall.shaper.traffic_shaper.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "exceed_class_id", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.firewall.traffic_class.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Exceed-Class-Id '{value}' not found in "
                "firewall/traffic-class"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_exceed_class_id_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "TrafficShaperModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:14.084106Z
# ============================================================================


"""
Pydantic Models for CMDB - emailfilter/block_allow_list

Runtime validation models for emailfilter/block_allow_list configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

class BlockAllowListEntriesTypeEnum(str, Enum):
    """Allowed values for type_ field in entries."""
    IP = "ip"
    EMAIL_TO = "email-to"
    EMAIL_FROM = "email-from"
    SUBJECT = "subject"

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class BlockAllowListEntries(BaseModel):
    """
    Child table model for entries.
    
    Anti-spam block/allow entries.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    status: Literal["enable", "disable"] = Field(default="enable", description="Enable/disable status.")    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Entry ID.")    
    type_: BlockAllowListEntriesTypeEnum = Field(default=BlockAllowListEntriesTypeEnum.IP, serialization_alias="type", description="Entry type.")    
    action: Literal["reject", "spam", "clear"] = Field(default="spam", description="Reject, mark as spam or good email.")    
    addr_type: Literal["ipv4", "ipv6"] = Field(default="ipv4", description="IP address type.")    
    ip4_subnet: str = Field(default="0.0.0.0 0.0.0.0", description="IPv4 network address/subnet mask bits.")    
    ip6_subnet: str = Field(default="::/128", description="IPv6 network address/subnet mask bits.")    
    pattern_type: Literal["wildcard", "regexp"] = Field(default="wildcard", description="Wildcard pattern or regular expression.")    
    pattern: str = Field(max_length=127, description="Pattern to match.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class BlockAllowListModel(BaseModel):
    """
    Pydantic model for emailfilter/block_allow_list configuration.
    
    Configure anti-spam block/allow list.
    
    Validation Rules:        - id_: min=0 max=4294967295 pattern=        - name: max_length=63 pattern=        - comment: max_length=255 pattern=        - entries: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    id_: int = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="ID.")    
    name: str = Field(max_length=63, description="Name of table.")    
    comment: str | None = Field(max_length=255, default=None, description="Optional comments.")    
    entries: list[BlockAllowListEntries] = Field(description="Anti-spam block/allow entries.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "BlockAllowListModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "BlockAllowListModel",    "BlockAllowListEntries",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.645988Z
# ============================================================================


"""Type stubs for WIFI category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .ap_channels import ApChannels
    from .ap_names import ApNames
    from .ap_status import ApStatus
    from .interfering_ap import InterferingAp
    from .matched_devices import MatchedDevices
    from .meta import Meta
    from .station_capability import StationCapability
    from .statistics import Statistics
    from .unassociated_devices import UnassociatedDevices
    from .ap_profile import ApProfile
    from .client_ns import Client
    from .euclid import Euclid
    from .firmware import Firmware
    from .managed_ap import ManagedAp
    from .nac_device import NacDevice
    from .network import Network
    from .region_image import RegionImage
    from .rogue_ap import RogueAp
    from .spectrum import Spectrum
    from .ssid import Ssid
    from .vlan_probe import VlanProbe

__all__ = [
    "ApChannels",
    "ApNames",
    "ApStatus",
    "InterferingAp",
    "MatchedDevices",
    "Meta",
    "StationCapability",
    "Statistics",
    "UnassociatedDevices",
    "Wifi",
]


class Wifi:
    """WIFI API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    ap_profile: ApProfile
    client: Client
    euclid: Euclid
    firmware: Firmware
    managed_ap: ManagedAp
    nac_device: NacDevice
    network: Network
    region_image: RegionImage
    rogue_ap: RogueAp
    spectrum: Spectrum
    ssid: Ssid
    vlan_probe: VlanProbe
    ap_channels: ApChannels
    ap_names: ApNames
    ap_status: ApStatus
    interfering_ap: InterferingAp
    matched_devices: MatchedDevices
    meta: Meta
    station_capability: StationCapability
    statistics: Statistics
    unassociated_devices: UnassociatedDevices

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize wifi category with HTTP client."""
        ...



"""FortiOS CMDB - HscalefwLicense category"""

from .upload import Upload

__all__ = [
    "HscalefwLicense",
    "Upload",
]


class HscalefwLicense:
    """HscalefwLicense endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """HscalefwLicense endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.upload = Upload(client)



"""FortiOS CMDB - IslLockdown category"""

from .status import Status
from .update import Update

__all__ = [
    "IslLockdown",
    "Status",
    "Update",
]


class IslLockdown:
    """IslLockdown endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """IslLockdown endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.status = Status(client)
        self.update = Update(client)



"""FortiOS CMDB - SecurityPolicy category"""

from ..security_policy_base import SecurityPolicy as SecurityPolicyBase
from .clear_counters import ClearCounters
from .update_global_label import UpdateGlobalLabel

__all__ = [
    "ClearCounters",
    "SecurityPolicy",
    "UpdateGlobalLabel",
]


class SecurityPolicy(SecurityPolicyBase):
    """SecurityPolicy endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """SecurityPolicy endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        super().__init__(client)  # Initialize base class with GET methods
        self.clear_counters = ClearCounters(client)
        self.update_global_label = UpdateGlobalLabel(client)



"""FortiOS CMDB - CentralSnatMap category (stub)"""

from typing import Any
from ..central_snat_map_base import CentralSnatMap as CentralSnatMapBase
from .clear_counters import ClearCounters
from .reset import Reset

class CentralSnatMap(CentralSnatMapBase):
    """CentralSnatMap endpoints wrapper for CMDB API."""

    clear_counters: ClearCounters
    reset: Reset

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: vpn/certificate/setting
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class SettingPayload(TypedDict, total=False):
    """Payload type for Setting operations."""
    ocsp_status: Literal["enable", "mandatory", "disable"]
    ocsp_option: Literal["certificate", "server"]
    proxy: str
    proxy_port: int
    proxy_username: str
    proxy_password: str
    source_ip: str
    ocsp_default_server: str
    interface_select_method: Literal["auto", "sdwan", "specify"]
    interface: str
    vrf_select: int
    check_ca_cert: Literal["enable", "disable"]
    check_ca_chain: Literal["enable", "disable"]
    subject_match: Literal["substring", "value"]
    subject_set: Literal["subset", "superset"]
    cn_match: Literal["substring", "value"]
    cn_allow_multi: Literal["disable", "enable"]
    crl_verification: str
    strict_ocsp_check: Literal["enable", "disable"]
    ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]
    cmp_save_extra_certs: Literal["enable", "disable"]
    cmp_key_usage_checking: Literal["enable", "disable"]
    cert_expire_warning: int
    certname_rsa1024: str
    certname_rsa2048: str
    certname_rsa4096: str
    certname_dsa1024: str
    certname_dsa2048: str
    certname_ecdsa256: str
    certname_ecdsa384: str
    certname_ecdsa521: str
    certname_ed25519: str
    certname_ed448: str


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class SettingResponse(TypedDict, total=False):
    """Response type for Setting - use with .dict property for typed dict access."""
    ocsp_status: Literal["enable", "mandatory", "disable"]
    ocsp_option: Literal["certificate", "server"]
    proxy: str
    proxy_port: int
    proxy_username: str
    proxy_password: str
    source_ip: str
    ocsp_default_server: str
    interface_select_method: Literal["auto", "sdwan", "specify"]
    interface: str
    vrf_select: int
    check_ca_cert: Literal["enable", "disable"]
    check_ca_chain: Literal["enable", "disable"]
    subject_match: Literal["substring", "value"]
    subject_set: Literal["subset", "superset"]
    cn_match: Literal["substring", "value"]
    cn_allow_multi: Literal["disable", "enable"]
    crl_verification: str
    strict_ocsp_check: Literal["enable", "disable"]
    ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]
    cmp_save_extra_certs: Literal["enable", "disable"]
    cmp_key_usage_checking: Literal["enable", "disable"]
    cert_expire_warning: int
    certname_rsa1024: str
    certname_rsa2048: str
    certname_rsa4096: str
    certname_dsa1024: str
    certname_dsa2048: str
    certname_ecdsa256: str
    certname_ecdsa384: str
    certname_ecdsa521: str
    certname_ed25519: str
    certname_ed448: str


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class SettingObject(FortiObject):
    """Typed FortiObject for Setting with field access."""
    ocsp_status: Literal["enable", "mandatory", "disable"]
    ocsp_option: Literal["certificate", "server"]
    proxy: str
    proxy_port: int
    proxy_username: str
    proxy_password: str
    source_ip: str
    ocsp_default_server: str
    interface_select_method: Literal["auto", "sdwan", "specify"]
    interface: str
    vrf_select: int
    check_ca_cert: Literal["enable", "disable"]
    check_ca_chain: Literal["enable", "disable"]
    subject_match: Literal["substring", "value"]
    subject_set: Literal["subset", "superset"]
    cn_match: Literal["substring", "value"]
    cn_allow_multi: Literal["disable", "enable"]
    crl_verification: str
    strict_ocsp_check: Literal["enable", "disable"]
    ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"]
    cmp_save_extra_certs: Literal["enable", "disable"]
    cmp_key_usage_checking: Literal["enable", "disable"]
    cert_expire_warning: int
    certname_rsa1024: str
    certname_rsa2048: str
    certname_rsa4096: str
    certname_dsa1024: str
    certname_dsa2048: str
    certname_ecdsa256: str
    certname_ecdsa384: str
    certname_ecdsa521: str
    certname_ed25519: str
    certname_ed448: str


# ================================================================
# Main Endpoint Class
# ================================================================

class Setting:
    """
    
    Endpoint: vpn/certificate/setting
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SettingObject: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: SettingPayload | None = ...,
        ocsp_status: Literal["enable", "mandatory", "disable"] | None = ...,
        ocsp_option: Literal["certificate", "server"] | None = ...,
        proxy: str | None = ...,
        proxy_port: int | None = ...,
        proxy_username: str | None = ...,
        proxy_password: str | None = ...,
        source_ip: str | None = ...,
        ocsp_default_server: str | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        check_ca_cert: Literal["enable", "disable"] | None = ...,
        check_ca_chain: Literal["enable", "disable"] | None = ...,
        subject_match: Literal["substring", "value"] | None = ...,
        subject_set: Literal["subset", "superset"] | None = ...,
        cn_match: Literal["substring", "value"] | None = ...,
        cn_allow_multi: Literal["disable", "enable"] | None = ...,
        crl_verification: str | None = ...,
        strict_ocsp_check: Literal["enable", "disable"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        cmp_save_extra_certs: Literal["enable", "disable"] | None = ...,
        cmp_key_usage_checking: Literal["enable", "disable"] | None = ...,
        cert_expire_warning: int | None = ...,
        certname_rsa1024: str | None = ...,
        certname_rsa2048: str | None = ...,
        certname_rsa4096: str | None = ...,
        certname_dsa1024: str | None = ...,
        certname_dsa2048: str | None = ...,
        certname_ecdsa256: str | None = ...,
        certname_ecdsa384: str | None = ...,
        certname_ecdsa521: str | None = ...,
        certname_ed25519: str | None = ...,
        certname_ed448: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> SettingObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: SettingPayload | None = ...,
        ocsp_status: Literal["enable", "mandatory", "disable"] | None = ...,
        ocsp_option: Literal["certificate", "server"] | None = ...,
        proxy: str | None = ...,
        proxy_port: int | None = ...,
        proxy_username: str | None = ...,
        proxy_password: str | None = ...,
        source_ip: str | None = ...,
        ocsp_default_server: str | None = ...,
        interface_select_method: Literal["auto", "sdwan", "specify"] | None = ...,
        interface: str | None = ...,
        vrf_select: int | None = ...,
        check_ca_cert: Literal["enable", "disable"] | None = ...,
        check_ca_chain: Literal["enable", "disable"] | None = ...,
        subject_match: Literal["substring", "value"] | None = ...,
        subject_set: Literal["subset", "superset"] | None = ...,
        cn_match: Literal["substring", "value"] | None = ...,
        cn_allow_multi: Literal["disable", "enable"] | None = ...,
        crl_verification: str | None = ...,
        strict_ocsp_check: Literal["enable", "disable"] | None = ...,
        ssl_min_proto_version: Literal["default", "SSLv3", "TLSv1", "TLSv1-1", "TLSv1-2", "TLSv1-3"] | None = ...,
        cmp_save_extra_certs: Literal["enable", "disable"] | None = ...,
        cmp_key_usage_checking: Literal["enable", "disable"] | None = ...,
        cert_expire_warning: int | None = ...,
        certname_rsa1024: str | None = ...,
        certname_rsa2048: str | None = ...,
        certname_rsa4096: str | None = ...,
        certname_dsa1024: str | None = ...,
        certname_dsa2048: str | None = ...,
        certname_ecdsa256: str | None = ...,
        certname_ecdsa384: str | None = ...,
        certname_ecdsa521: str | None = ...,
        certname_ed25519: str | None = ...,
        certname_ed448: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Setting",
    "SettingPayload",
    "SettingResponse",
    "SettingObject",
]


"""FortiOS CMDB - Vmlicense category"""

from .download import Download
from .download_eval import DownloadEval
from .upload import Upload

__all__ = [
    "Download",
    "DownloadEval",
    "Upload",
    "Vmlicense",
]


class Vmlicense:
    """Vmlicense endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Vmlicense endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.download = Download(client)
        self.download_eval = DownloadEval(client)
        self.upload = Upload(client)



"""FortiOS CMDB - Shaper category (stub)"""

from typing import Any
from ..shaper_base import Shaper as ShaperBase
from .multi_class_shaper import MultiClassShaper
from .reset import Reset

class Shaper(ShaperBase):
    """Shaper endpoints wrapper for CMDB API."""

    multi_class_shaper: MultiClassShaper
    reset: Reset

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



"""Type stubs for SCHEDULE category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .group import Group
    from .onetime import Onetime
    from .recurring import Recurring

__all__ = [
    "Group",
    "Onetime",
    "Recurring",
    "Schedule",
]


class Schedule:
    """SCHEDULE API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    group: Group
    onetime: Onetime
    recurring: Recurring

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize schedule category with HTTP client."""
        ...



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: user/group
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class GroupMemberItem:
    """Nested item for member field - supports attribute access."""
    name: str


class GroupMatchItem:
    """Nested item for match field - supports attribute access."""
    id: int
    server_name: str
    group_name: str


class GroupGuestItem:
    """Nested item for guest field - supports attribute access."""
    id: int
    user_id: str
    name: str
    password: str
    mobile_phone: str
    sponsor: str
    company: str
    email: str
    expiration: str
    comment: str


class GroupPayload(TypedDict, total=False):
    """Payload type for Group operations."""
    name: str
    id: int
    group_type: Literal["firewall", "fsso-service", "rsso", "guest"]
    authtimeout: int
    auth_concurrent_override: Literal["enable", "disable"]
    auth_concurrent_value: int
    http_digest_realm: str
    sso_attribute_value: str
    member: str | list[str] | list[dict[str, Any]] | list[GroupMemberItem]
    match: str | list[str] | list[dict[str, Any]] | list[GroupMatchItem]
    user_id: Literal["email", "auto-generate", "specify"]
    password: Literal["auto-generate", "specify", "disable"]
    user_name: Literal["disable", "enable"]
    sponsor: Literal["optional", "mandatory", "disabled"]
    company: Literal["optional", "mandatory", "disabled"]
    email: Literal["disable", "enable"]
    mobile_phone: Literal["disable", "enable"]
    sms_server: Literal["fortiguard", "custom"]
    sms_custom_server: str
    expire_type: Literal["immediately", "first-successful-login"]
    expire: int
    max_accounts: int
    multiple_guest_add: Literal["disable", "enable"]
    guest: str | list[str] | list[dict[str, Any]] | list[GroupGuestItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class GroupResponse(TypedDict, total=False):
    """Response type for Group - use with .dict property for typed dict access."""
    name: str
    id: int
    group_type: Literal["firewall", "fsso-service", "rsso", "guest"]
    authtimeout: int
    auth_concurrent_override: Literal["enable", "disable"]
    auth_concurrent_value: int
    http_digest_realm: str
    sso_attribute_value: str
    member: list[GroupMemberItem]
    match: list[GroupMatchItem]
    user_id: Literal["email", "auto-generate", "specify"]
    password: Literal["auto-generate", "specify", "disable"]
    user_name: Literal["disable", "enable"]
    sponsor: Literal["optional", "mandatory", "disabled"]
    company: Literal["optional", "mandatory", "disabled"]
    email: Literal["disable", "enable"]
    mobile_phone: Literal["disable", "enable"]
    sms_server: Literal["fortiguard", "custom"]
    sms_custom_server: str
    expire_type: Literal["immediately", "first-successful-login"]
    expire: int
    max_accounts: int
    multiple_guest_add: Literal["disable", "enable"]
    guest: list[GroupGuestItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class GroupObject(FortiObject):
    """Typed FortiObject for Group with field access."""
    name: str
    id: int
    group_type: Literal["firewall", "fsso-service", "rsso", "guest"]
    authtimeout: int
    auth_concurrent_override: Literal["enable", "disable"]
    auth_concurrent_value: int
    http_digest_realm: str
    sso_attribute_value: str
    member: list[GroupMemberItem]
    match: list[GroupMatchItem]
    user_id: Literal["email", "auto-generate", "specify"]
    password: Literal["auto-generate", "specify", "disable"]
    user_name: Literal["disable", "enable"]
    sponsor: Literal["optional", "mandatory", "disabled"]
    company: Literal["optional", "mandatory", "disabled"]
    email: Literal["disable", "enable"]
    mobile_phone: Literal["disable", "enable"]
    sms_server: Literal["fortiguard", "custom"]
    sms_custom_server: str
    expire_type: Literal["immediately", "first-successful-login"]
    expire: int
    max_accounts: int
    multiple_guest_add: Literal["disable", "enable"]
    guest: list[GroupGuestItem]


# ================================================================
# Main Endpoint Class
# ================================================================

class Group:
    """
    
    Endpoint: user/group
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> GroupObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[GroupObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[dict[str, Any]] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[dict[str, Any]] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[dict[str, Any]] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> GroupObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[dict[str, Any]] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[dict[str, Any]] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[dict[str, Any]] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> GroupObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: GroupPayload | None = ...,
        name: str | None = ...,
        id: int | None = ...,
        group_type: Literal["firewall", "fsso-service", "rsso", "guest"] | None = ...,
        authtimeout: int | None = ...,
        auth_concurrent_override: Literal["enable", "disable"] | None = ...,
        auth_concurrent_value: int | None = ...,
        http_digest_realm: str | None = ...,
        sso_attribute_value: str | None = ...,
        member: str | list[str] | list[dict[str, Any]] | list[GroupMemberItem] | None = ...,
        match: str | list[str] | list[dict[str, Any]] | list[GroupMatchItem] | None = ...,
        user_id: Literal["email", "auto-generate", "specify"] | None = ...,
        password: Literal["auto-generate", "specify", "disable"] | None = ...,
        user_name: Literal["disable", "enable"] | None = ...,
        sponsor: Literal["optional", "mandatory", "disabled"] | None = ...,
        company: Literal["optional", "mandatory", "disabled"] | None = ...,
        email: Literal["disable", "enable"] | None = ...,
        mobile_phone: Literal["disable", "enable"] | None = ...,
        sms_server: Literal["fortiguard", "custom"] | None = ...,
        sms_custom_server: str | None = ...,
        expire_type: Literal["immediately", "first-successful-login"] | None = ...,
        expire: int | None = ...,
        max_accounts: int | None = ...,
        multiple_guest_add: Literal["disable", "enable"] | None = ...,
        guest: str | list[str] | list[dict[str, Any]] | list[GroupGuestItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Group",
    "GroupPayload",
    "GroupResponse",
    "GroupObject",
]


"""
Central helpers module for hfortix_fortios package.

This module consolidates all helper functions used across the package:
- API endpoints (hfortix_fortios.api.v2.cmdb.*, monitor.*)
- Convenience wrappers (hfortix_fortios.firewall.*)
- Future modules (system, user, router, etc.)

Organized into logical submodules:
- builders: Payload building (build_cmdb_payload,
    build_cmdb_payload_normalized)
- normalizers: List normalization (normalize_to_name_list,
    normalize_member_list)
- validators: All validation functions (generic + domain-specific)
- converters: Type conversion and data cleaning
- response: Response parsing helpers


Import from this module for consistency across the codebase:
    from hfortix_fortios._helpers import build_cmdb_payload, validate_color
"""

# Payload builders
from hfortix_fortios._helpers.builders import (
    build_api_payload,
    build_cmdb_payload,
    build_cmdb_payload_normalized,
)

# Data converters and cleaners
from hfortix_fortios._helpers.converters import (
    convert_boolean_to_str,
    filter_empty_values,
    quote_path_param,
)

# Metadata accessors (shared by all validator modules)
from hfortix_fortios._helpers.metadata import (
    get_all_fields,
    get_field_constraints,
    get_field_default,
    get_field_description,
    get_field_metadata,
    get_field_options,
    get_field_type,
    get_nested_schema,
    validate_field_value,
)

# Metadata mixin for endpoint classes
from hfortix_fortios._helpers.metadata_mixin import MetadataMixin

# List normalizers
from hfortix_fortios._helpers.normalizers import (
    normalize_day_field,
    normalize_member_list,
    normalize_table_field,
    normalize_to_name_list,
)

# Response helpers
from hfortix_fortios._helpers.response import (
    get_mkey,
    get_name,
    get_results,
    is_success,
)

# Central validation functions (used by all endpoint validators)
from hfortix_fortios._helpers.validation import (
    validate_enum_field,
    validate_multiple_enums,
    validate_multiple_query_params,
    validate_query_parameter,
)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields_central,
)

# Validators - SSH/SSL proxy-specific
# Validators - Firewall-specific
# Validators - Generic
from hfortix_fortios._helpers.validators import (
    validate_address_pairs,
    validate_color,
    validate_day_names,
    validate_enable_disable,
    validate_integer_range,
    validate_ip_address,
    validate_ip_network,
    validate_ipv6_address,
    validate_mac_address,
    validate_policy_id,
    validate_port_number,
    validate_required_fields,
    validate_schedule_name,
    validate_seq_num,
    validate_ssh_host_key_nid,
    validate_ssh_host_key_status,
    validate_ssh_host_key_type,
    validate_ssh_host_key_usage,
    validate_ssh_source,
    validate_ssl_cipher_action,
    validate_ssl_dh_bits,
    validate_status,
    validate_string_length,
    validate_time_format,
)

__all__ = [
    # Payload building
    "build_api_payload",
    "build_cmdb_payload",
    "build_cmdb_payload_normalized",
    # List normalization
    "normalize_to_name_list",
    "normalize_member_list",
    "normalize_table_field",
    # Data cleaning and conversion
    "filter_empty_values",
    "convert_boolean_to_str",
    "quote_path_param",
    # Response helpers
    "get_name",
    "get_mkey",  # Alias for backward compatibility
    "get_results",
    "is_success",
    # Metadata accessors
    "get_field_description",
    "get_field_type",
    "get_field_constraints",
    "get_field_default",
    "get_field_options",
    "get_nested_schema",
    "get_all_fields",
    "get_field_metadata",
    "validate_field_value",
    # Metadata mixin
    "MetadataMixin",
    # Central validation functions
    "_validate_required_fields_central",
    "validate_enum_field",
    "validate_query_parameter",
    "validate_multiple_enums",
    "validate_multiple_query_params",
    # Validation - Generic
    "validate_required_fields",
    "validate_color",
    "validate_status",
    "validate_mac_address",
    "validate_ip_address",
    "validate_ipv6_address",
    "validate_ip_network",
    "validate_string_length",
    "validate_integer_range",
    "validate_port_number",
    "validate_enable_disable",
    # Validation - Firewall-specific
    "validate_policy_id",
    "validate_address_pairs",
    "validate_seq_num",
    "validate_schedule_name",
    "validate_time_format",
    "validate_day_names",
    # Validation - SSH/SSL proxy-specific
    "validate_ssh_host_key_type",
    "validate_ssh_host_key_status",
    "validate_ssh_host_key_nid",
    "validate_ssh_host_key_usage",
    "validate_ssh_source",
    "validate_ssl_dh_bits",
    "validate_ssl_cipher_action",
]



"""FortiOS CMDB - UsbLog category"""

from ..usb_log_base import UsbLog as UsbLogBase
from .start import Start
from .stop import Stop

__all__ = [
    "Start",
    "Stop",
    "UsbLog",
]


class UsbLog(UsbLogBase):
    """UsbLog endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """UsbLog endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        super().__init__(client)  # Initialize base class with GET methods
        self.start = Start(client)
        self.stop = Stop(client)



"""
Pydantic Models for CMDB - switch_controller/lldp_profile

Runtime validation models for switch_controller/lldp_profile configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class LldpProfileMedNetworkPolicy(BaseModel):
    """
    Child table model for med-network-policy.
    
    Configuration method to edit Media Endpoint Discovery (MED) network policy type-length-value (TLV) categories.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=63, default=None, description="Policy type name.")    
    status: Literal["disable", "enable"] | None = Field(default="disable", description="Enable or disable this TLV.")    
    vlan_intf: str | None = Field(max_length=15, default=None, description="VLAN interface to advertise; if configured on port.")  # datasource: ['system.interface.name']    
    assign_vlan: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable VLAN assignment when this profile is applied on managed FortiSwitch port.")    
    priority: int | None = Field(ge=0, le=7, default=0, description="Advertised Layer 2 priority (0 - 7; from lowest to highest priority).")    
    dscp: int | None = Field(ge=0, le=63, default=0, description="Advertised Differentiated Services Code Point (DSCP) value, a packet header value indicating the level of service requested for traffic, such as high priority or best effort delivery.")
class LldpProfileMedLocationService(BaseModel):
    """
    Child table model for med-location-service.
    
    Configuration method to edit Media Endpoint Discovery (MED) location service type-length-value (TLV) categories.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=63, default=None, description="Location service type name.")    
    status: Literal["disable", "enable"] | None = Field(default="disable", description="Enable or disable this TLV.")    
    sys_location_id: str | None = Field(max_length=63, default=None, description="Location service ID.")  # datasource: ['switch-controller.location.name']
class LldpProfileCustomTlvs(BaseModel):
    """
    Child table model for custom-tlvs.
    
    Configuration method to edit custom TLV entries.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    name: str | None = Field(max_length=63, default=None, description="TLV name (not sent).")    
    oui: str = Field(default="000000", description="Organizationally unique identifier (OUI), a 3-byte hexadecimal number, for this TLV.")    
    subtype: int | None = Field(ge=0, le=255, default=0, description="Organizationally defined subtype (0 - 255).")    
    information_string: str | None = Field(default=None, description="Organizationally defined information string (0 - 507 hexadecimal bytes).")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class LldpProfileMedTlvsEnum(str, Enum):
    """Allowed values for med_tlvs field."""
    INVENTORY_MANAGEMENT = "inventory-management"
    NETWORK_POLICY = "network-policy"
    POWER_MANAGEMENT = "power-management"
    LOCATION_IDENTIFICATION = "location-identification"


# ============================================================================
# Main Model
# ============================================================================

class LldpProfileModel(BaseModel):
    """
    Pydantic model for switch_controller/lldp_profile configuration.
    
    Configure FortiSwitch LLDP profiles.
    
    Validation Rules:        - name: max_length=63 pattern=        - med_tlvs: pattern=        - _8021_tlvs: pattern=        - _8023_tlvs: pattern=        - auto_isl: pattern=        - auto_isl_hello_timer: min=1 max=30 pattern=        - auto_isl_receive_timeout: min=0 max=90 pattern=        - auto_isl_port_group: min=0 max=9 pattern=        - auto_mclag_icl: pattern=        - auto_isl_auth: pattern=        - auto_isl_auth_user: max_length=63 pattern=        - auto_isl_auth_identity: max_length=63 pattern=        - auto_isl_auth_reauth: min=180 max=3600 pattern=        - auto_isl_auth_encrypt: pattern=        - auto_isl_auth_macsec_profile: max_length=63 pattern=        - med_network_policy: pattern=        - med_location_service: pattern=        - custom_tlvs: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=63, default=None, description="Profile name.")    
    med_tlvs: list[LldpProfileMedTlvsEnum] = Field(default_factory=list, description="Transmitted LLDP-MED TLVs (type-length-value descriptions).")    
    _8021_tlvs: list[Literal["port-vlan-id"]] = Field(default_factory=list, serialization_alias="802.1-tlvs", description="Transmitted IEEE 802.1 TLVs.")    
    _8023_tlvs: list[Literal["max-frame-size", "power-negotiation"]] = Field(default_factory=list, serialization_alias="802.3-tlvs", description="Transmitted IEEE 802.3 TLVs.")    
    auto_isl: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable auto inter-switch LAG.")    
    auto_isl_hello_timer: int | None = Field(ge=1, le=30, default=3, description="Auto inter-switch LAG hello timer duration (1 - 30 sec, default = 3).")    
    auto_isl_receive_timeout: int | None = Field(ge=0, le=90, default=60, description="Auto inter-switch LAG timeout if no response is received (3 - 90 sec, default = 9).")    
    auto_isl_port_group: int | None = Field(ge=0, le=9, default=0, description="Auto inter-switch LAG port group ID (0 - 9).")    
    auto_mclag_icl: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable MCLAG inter chassis link.")    
    auto_isl_auth: Literal["legacy", "strict", "relax"] | None = Field(default="legacy", description="Auto inter-switch LAG authentication mode.")    
    auto_isl_auth_user: str | None = Field(max_length=63, default=None, description="Auto inter-switch LAG authentication user certificate.")    
    auto_isl_auth_identity: str | None = Field(max_length=63, default=None, description="Auto inter-switch LAG authentication identity.")    
    auto_isl_auth_reauth: int | None = Field(ge=180, le=3600, default=3600, description="Auto inter-switch LAG authentication reauth period in seconds(10 - 3600, default = 3600).")    
    auto_isl_auth_encrypt: Literal["none", "mixed", "must"] | None = Field(default="none", description="Auto inter-switch LAG encryption mode.")    
    auto_isl_auth_macsec_profile: str | None = Field(max_length=63, default=None, description="Auto inter-switch LAG macsec profile for encryption.")    
    med_network_policy: list[LldpProfileMedNetworkPolicy] = Field(default_factory=list, description="Configuration method to edit Media Endpoint Discovery (MED) network policy type-length-value (TLV) categories.")    
    med_location_service: list[LldpProfileMedLocationService] = Field(default_factory=list, description="Configuration method to edit Media Endpoint Discovery (MED) location service type-length-value (TLV) categories.")    
    custom_tlvs: list[LldpProfileCustomTlvs] = Field(default_factory=list, description="Configuration method to edit custom TLV entries.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "LldpProfileModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_med_network_policy_references(self, client: Any) -> list[str]:
        """
        Validate med_network_policy references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = LldpProfileModel(
            ...     med_network_policy=[{"vlan-intf": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_med_network_policy_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.lldp_profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "med_network_policy", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("vlan-intf")
            else:
                value = getattr(item, "vlan-intf", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.system.interface.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Med-Network-Policy '{value}' not found in "
                    "system/interface"
                )        
        return errors    
    async def validate_med_location_service_references(self, client: Any) -> list[str]:
        """
        Validate med_location_service references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - switch-controller/location        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = LldpProfileModel(
            ...     med_location_service=[{"sys-location-id": "invalid-name"}],
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_med_location_service_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.lldp_profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate child table items
        values = getattr(self, "med_location_service", [])
        if not values:
            return errors
        
        for item in values:
            if isinstance(item, dict):
                value = item.get("sys-location-id")
            else:
                value = getattr(item, "sys-location-id", None)
            
            if not value:
                continue
            
            # Check all datasource endpoints
            found = False
            if await client.api.cmdb.switch_controller.location.exists(value):
                found = True
            
            if not found:
                errors.append(
                    f"Med-Location-Service '{value}' not found in "
                    "switch-controller/location"
                )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_med_network_policy_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_med_location_service_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "LldpProfileModel",    "LldpProfileMedNetworkPolicy",    "LldpProfileMedLocationService",    "LldpProfileCustomTlvs",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.891368Z
# ============================================================================


"""FortiOS CMDB - Interface category (stub)"""

from typing import Any
from ..interface_base import Interface as InterfaceBase
from .dhcp_renew import DhcpRenew
from .dhcp_status import DhcpStatus
from .kernel_interfaces import KernelInterfaces
from .poe import Poe
from .poe_usage import PoeUsage
from .speed_test_status import SpeedTestStatus
from .speed_test_trigger import SpeedTestTrigger
from .transceivers import Transceivers
from .wake_on_lan import WakeOnLan

class Interface(InterfaceBase):
    """Interface endpoints wrapper for CMDB API."""

    dhcp_renew: DhcpRenew
    dhcp_status: DhcpStatus
    kernel_interfaces: KernelInterfaces
    poe: Poe
    poe_usage: PoeUsage
    speed_test_status: SpeedTestStatus
    speed_test_trigger: SpeedTestTrigger
    transceivers: Transceivers
    wake_on_lan: WakeOnLan

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: firewall/central_snat_map
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class CentralSnatMapSrcintfItem:
    """Nested item for srcintf field - supports attribute access."""
    name: str


class CentralSnatMapDstintfItem:
    """Nested item for dstintf field - supports attribute access."""
    name: str


class CentralSnatMapOrigaddrItem:
    """Nested item for orig-addr field - supports attribute access."""
    name: str


class CentralSnatMapOrigaddr6Item:
    """Nested item for orig-addr6 field - supports attribute access."""
    name: str


class CentralSnatMapDstaddrItem:
    """Nested item for dst-addr field - supports attribute access."""
    name: str


class CentralSnatMapDstaddr6Item:
    """Nested item for dst-addr6 field - supports attribute access."""
    name: str


class CentralSnatMapNatippoolItem:
    """Nested item for nat-ippool field - supports attribute access."""
    name: str


class CentralSnatMapNatippool6Item:
    """Nested item for nat-ippool6 field - supports attribute access."""
    name: str


class CentralSnatMapPayload(TypedDict, total=False):
    """Payload type for CentralSnatMap operations."""
    policyid: int
    uuid: str
    status: Literal["enable", "disable"]
    type: Literal["ipv4", "ipv6"]
    srcintf: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapSrcintfItem]
    dstintf: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstintfItem]
    orig_addr: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapOrigaddrItem]
    orig_addr6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapOrigaddr6Item]
    dst_addr: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstaddrItem]
    dst_addr6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstaddr6Item]
    protocol: int
    orig_port: str
    nat: Literal["disable", "enable"]
    nat46: Literal["enable", "disable"]
    nat64: Literal["enable", "disable"]
    nat_ippool: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapNatippoolItem]
    nat_ippool6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapNatippool6Item]
    port_preserve: Literal["enable", "disable"]
    port_random: Literal["enable", "disable"]
    nat_port: str
    dst_port: str
    comments: str


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class CentralSnatMapResponse(TypedDict, total=False):
    """Response type for CentralSnatMap - use with .dict property for typed dict access."""
    policyid: int
    uuid: str
    status: Literal["enable", "disable"]
    type: Literal["ipv4", "ipv6"]
    srcintf: list[CentralSnatMapSrcintfItem]
    dstintf: list[CentralSnatMapDstintfItem]
    orig_addr: list[CentralSnatMapOrigaddrItem]
    orig_addr6: list[CentralSnatMapOrigaddr6Item]
    dst_addr: list[CentralSnatMapDstaddrItem]
    dst_addr6: list[CentralSnatMapDstaddr6Item]
    protocol: int
    orig_port: str
    nat: Literal["disable", "enable"]
    nat46: Literal["enable", "disable"]
    nat64: Literal["enable", "disable"]
    nat_ippool: list[CentralSnatMapNatippoolItem]
    nat_ippool6: list[CentralSnatMapNatippool6Item]
    port_preserve: Literal["enable", "disable"]
    port_random: Literal["enable", "disable"]
    nat_port: str
    dst_port: str
    comments: str


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class CentralSnatMapObject(FortiObject):
    """Typed FortiObject for CentralSnatMap with field access."""
    policyid: int
    uuid: str
    status: Literal["enable", "disable"]
    type: Literal["ipv4", "ipv6"]
    srcintf: list[CentralSnatMapSrcintfItem]
    dstintf: list[CentralSnatMapDstintfItem]
    orig_addr: list[CentralSnatMapOrigaddrItem]
    orig_addr6: list[CentralSnatMapOrigaddr6Item]
    dst_addr: list[CentralSnatMapDstaddrItem]
    dst_addr6: list[CentralSnatMapDstaddr6Item]
    protocol: int
    orig_port: str
    nat: Literal["disable", "enable"]
    nat46: Literal["enable", "disable"]
    nat64: Literal["enable", "disable"]
    nat_ippool: list[CentralSnatMapNatippoolItem]
    nat_ippool6: list[CentralSnatMapNatippool6Item]
    port_preserve: Literal["enable", "disable"]
    port_random: Literal["enable", "disable"]
    nat_port: str
    dst_port: str
    comments: str


# ================================================================
# Main Endpoint Class
# ================================================================

class CentralSnatMap:
    """
    
    Endpoint: firewall/central_snat_map
    Category: cmdb
    MKey: policyid
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        policyid: int,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> CentralSnatMapObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[CentralSnatMapObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: CentralSnatMapPayload | None = ...,
        policyid: int | None = ...,
        uuid: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        type: Literal["ipv4", "ipv6"] | None = ...,
        srcintf: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapSrcintfItem] | None = ...,
        dstintf: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstintfItem] | None = ...,
        orig_addr: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapOrigaddrItem] | None = ...,
        orig_addr6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapOrigaddr6Item] | None = ...,
        dst_addr: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstaddrItem] | None = ...,
        dst_addr6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstaddr6Item] | None = ...,
        protocol: int | None = ...,
        orig_port: str | None = ...,
        nat: Literal["disable", "enable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        nat_ippool: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapNatippoolItem] | None = ...,
        nat_ippool6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapNatippool6Item] | None = ...,
        port_preserve: Literal["enable", "disable"] | None = ...,
        port_random: Literal["enable", "disable"] | None = ...,
        nat_port: str | None = ...,
        dst_port: str | None = ...,
        comments: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> CentralSnatMapObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: CentralSnatMapPayload | None = ...,
        policyid: int | None = ...,
        uuid: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        type: Literal["ipv4", "ipv6"] | None = ...,
        srcintf: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapSrcintfItem] | None = ...,
        dstintf: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstintfItem] | None = ...,
        orig_addr: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapOrigaddrItem] | None = ...,
        orig_addr6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapOrigaddr6Item] | None = ...,
        dst_addr: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstaddrItem] | None = ...,
        dst_addr6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstaddr6Item] | None = ...,
        protocol: int | None = ...,
        orig_port: str | None = ...,
        nat: Literal["disable", "enable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        nat_ippool: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapNatippoolItem] | None = ...,
        nat_ippool6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapNatippool6Item] | None = ...,
        port_preserve: Literal["enable", "disable"] | None = ...,
        port_random: Literal["enable", "disable"] | None = ...,
        nat_port: str | None = ...,
        dst_port: str | None = ...,
        comments: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> CentralSnatMapObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        policyid: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        policyid: int,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: CentralSnatMapPayload | None = ...,
        policyid: int | None = ...,
        uuid: str | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        type: Literal["ipv4", "ipv6"] | None = ...,
        srcintf: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapSrcintfItem] | None = ...,
        dstintf: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstintfItem] | None = ...,
        orig_addr: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapOrigaddrItem] | None = ...,
        orig_addr6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapOrigaddr6Item] | None = ...,
        dst_addr: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstaddrItem] | None = ...,
        dst_addr6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapDstaddr6Item] | None = ...,
        protocol: int | None = ...,
        orig_port: str | None = ...,
        nat: Literal["disable", "enable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        nat_ippool: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapNatippoolItem] | None = ...,
        nat_ippool6: str | list[str] | list[dict[str, Any]] | list[CentralSnatMapNatippool6Item] | None = ...,
        port_preserve: Literal["enable", "disable"] | None = ...,
        port_random: Literal["enable", "disable"] | None = ...,
        nat_port: str | None = ...,
        dst_port: str | None = ...,
        comments: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "CentralSnatMap",
    "CentralSnatMapPayload",
    "CentralSnatMapResponse",
    "CentralSnatMapObject",
]


"""FortiOS CMDB - Extender category (stub)"""

from typing import Any
from ..extender_base import Extender as ExtenderBase
from .diagnose import Diagnose
from .modem_firmware import ModemFirmware
from .reset import Reset
from .upgrade import Upgrade

class Extender(ExtenderBase):
    """Extender endpoints wrapper for CMDB API."""

    diagnose: Diagnose
    modem_firmware: ModemFirmware
    reset: Reset
    upgrade: Upgrade

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...



"""Validation helpers for system/pppoe_interface - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "device",  # Name for the physical interface.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "name": "",
    "dial-on-demand": "disable",
    "ipv6": "disable",
    "device": "",
    "username": "",
    "pppoe-egress-cos": "cos0",
    "auth-type": "auto",
    "ipunnumbered": "0.0.0.0",
    "pppoe-unnumbered-negotiate": "enable",
    "idle-timeout": 0,
    "multilink": "disable",
    "mrru": 1500,
    "disc-retry-timeout": 1,
    "padt-retry-timeout": 1,
    "service-name": "",
    "ac-name": "",
    "lcp-echo-interval": 5,
    "lcp-max-echo-fails": 3,
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "name": "string",  # Name of the PPPoE interface.
    "dial-on-demand": "option",  # Enable/disable dial on demand to dial the PPPoE interface wh
    "ipv6": "option",  # Enable/disable IPv6 Control Protocol (IPv6CP).
    "device": "string",  # Name for the physical interface.
    "username": "string",  # User name.
    "password": "password",  # Enter the password.
    "pppoe-egress-cos": "option",  # CoS in VLAN tag for outgoing PPPoE/PPP packets.
    "auth-type": "option",  # PPP authentication type to use.
    "ipunnumbered": "ipv4-address",  # PPPoE unnumbered IP.
    "pppoe-unnumbered-negotiate": "option",  # Enable/disable PPPoE unnumbered negotiation.
    "idle-timeout": "integer",  # PPPoE auto disconnect after idle timeout (0-4294967295 sec).
    "multilink": "option",  # Enable/disable PPP multilink support.
    "mrru": "integer",  # PPP MRRU (296 - 65535, default = 1500).
    "disc-retry-timeout": "integer",  # PPPoE discovery init timeout value in (0-4294967295 sec).
    "padt-retry-timeout": "integer",  # PPPoE terminate timeout value in (0-4294967295 sec).
    "service-name": "string",  # PPPoE service name.
    "ac-name": "string",  # PPPoE AC name.
    "lcp-echo-interval": "integer",  # Time in seconds between PPPoE Link Control Protocol (LCP) ec
    "lcp-max-echo-fails": "integer",  # Maximum missed LCP echo messages before disconnect.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "name": "Name of the PPPoE interface.",
    "dial-on-demand": "Enable/disable dial on demand to dial the PPPoE interface when packets are routed to the PPPoE interface.",
    "ipv6": "Enable/disable IPv6 Control Protocol (IPv6CP).",
    "device": "Name for the physical interface.",
    "username": "User name.",
    "password": "Enter the password.",
    "pppoe-egress-cos": "CoS in VLAN tag for outgoing PPPoE/PPP packets.",
    "auth-type": "PPP authentication type to use.",
    "ipunnumbered": "PPPoE unnumbered IP.",
    "pppoe-unnumbered-negotiate": "Enable/disable PPPoE unnumbered negotiation.",
    "idle-timeout": "PPPoE auto disconnect after idle timeout (0-4294967295 sec).",
    "multilink": "Enable/disable PPP multilink support.",
    "mrru": "PPP MRRU (296 - 65535, default = 1500).",
    "disc-retry-timeout": "PPPoE discovery init timeout value in (0-4294967295 sec).",
    "padt-retry-timeout": "PPPoE terminate timeout value in (0-4294967295 sec).",
    "service-name": "PPPoE service name.",
    "ac-name": "PPPoE AC name.",
    "lcp-echo-interval": "Time in seconds between PPPoE Link Control Protocol (LCP) echo requests.",
    "lcp-max-echo-fails": "Maximum missed LCP echo messages before disconnect.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "name": {"type": "string", "max_length": 15},
    "device": {"type": "string", "max_length": 15},
    "username": {"type": "string", "max_length": 64},
    "idle-timeout": {"type": "integer", "min": 0, "max": 4294967295},
    "mrru": {"type": "integer", "min": 296, "max": 65535},
    "disc-retry-timeout": {"type": "integer", "min": 0, "max": 4294967295},
    "padt-retry-timeout": {"type": "integer", "min": 0, "max": 4294967295},
    "service-name": {"type": "string", "max_length": 63},
    "ac-name": {"type": "string", "max_length": 63},
    "lcp-echo-interval": {"type": "integer", "min": 0, "max": 32767},
    "lcp-max-echo-fails": {"type": "integer", "min": 0, "max": 32767},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
}


# Valid enum values from API documentation
VALID_BODY_DIAL_ON_DEMAND = [
    "enable",
    "disable",
]
VALID_BODY_IPV6 = [
    "enable",
    "disable",
]
VALID_BODY_PPPOE_EGRESS_COS = [
    "cos0",
    "cos1",
    "cos2",
    "cos3",
    "cos4",
    "cos5",
    "cos6",
    "cos7",
]
VALID_BODY_AUTH_TYPE = [
    "auto",
    "pap",
    "chap",
    "mschapv1",
    "mschapv2",
]
VALID_BODY_PPPOE_UNNUMBERED_NEGOTIATE = [
    "enable",
    "disable",
]
VALID_BODY_MULTILINK = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_system_pppoe_interface_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for system/pppoe_interface."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_system_pppoe_interface_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new system/pppoe_interface object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "dial-on-demand" in payload:
        is_valid, error = _validate_enum_field(
            "dial-on-demand",
            payload["dial-on-demand"],
            VALID_BODY_DIAL_ON_DEMAND,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ipv6" in payload:
        is_valid, error = _validate_enum_field(
            "ipv6",
            payload["ipv6"],
            VALID_BODY_IPV6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "pppoe-egress-cos" in payload:
        is_valid, error = _validate_enum_field(
            "pppoe-egress-cos",
            payload["pppoe-egress-cos"],
            VALID_BODY_PPPOE_EGRESS_COS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-type" in payload:
        is_valid, error = _validate_enum_field(
            "auth-type",
            payload["auth-type"],
            VALID_BODY_AUTH_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "pppoe-unnumbered-negotiate" in payload:
        is_valid, error = _validate_enum_field(
            "pppoe-unnumbered-negotiate",
            payload["pppoe-unnumbered-negotiate"],
            VALID_BODY_PPPOE_UNNUMBERED_NEGOTIATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "multilink" in payload:
        is_valid, error = _validate_enum_field(
            "multilink",
            payload["multilink"],
            VALID_BODY_MULTILINK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_system_pppoe_interface_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update system/pppoe_interface."""
    # Validate enum values using central function
    if "dial-on-demand" in payload:
        is_valid, error = _validate_enum_field(
            "dial-on-demand",
            payload["dial-on-demand"],
            VALID_BODY_DIAL_ON_DEMAND,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "ipv6" in payload:
        is_valid, error = _validate_enum_field(
            "ipv6",
            payload["ipv6"],
            VALID_BODY_IPV6,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "pppoe-egress-cos" in payload:
        is_valid, error = _validate_enum_field(
            "pppoe-egress-cos",
            payload["pppoe-egress-cos"],
            VALID_BODY_PPPOE_EGRESS_COS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "auth-type" in payload:
        is_valid, error = _validate_enum_field(
            "auth-type",
            payload["auth-type"],
            VALID_BODY_AUTH_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "pppoe-unnumbered-negotiate" in payload:
        is_valid, error = _validate_enum_field(
            "pppoe-unnumbered-negotiate",
            payload["pppoe-unnumbered-negotiate"],
            VALID_BODY_PPPOE_UNNUMBERED_NEGOTIATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "multilink" in payload:
        is_valid, error = _validate_enum_field(
            "multilink",
            payload["multilink"],
            VALID_BODY_MULTILINK,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "system/pppoe_interface",
    "category": "cmdb",
    "api_path": "system/pppoe-interface",
    "mkey": "name",
    "mkey_type": "string",
    "help": "Configure the PPPoE interfaces.",
    "total_fields": 19,
    "required_fields_count": 1,
    "fields_with_defaults_count": 18,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""
Pydantic Models for CMDB - system/ngfw_settings

Runtime validation models for system/ngfw_settings configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class NgfwSettingsModel(BaseModel):
    """
    Pydantic model for system/ngfw_settings configuration.
    
    Configure IPS NGFW policy-mode VDOM settings.
    
    Validation Rules:        - match_timeout: min=0 max=1800 pattern=        - tcp_match_timeout: min=0 max=1800 pattern=        - tcp_halfopen_match_timeout: min=0 max=300 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    match_timeout: int | None = Field(ge=0, le=1800, default=300, description="Number of seconds to wait before a security policy match for an idle non-TCP session (0 - 1800, default = 300, 0 means unlimited).")    
    tcp_match_timeout: int | None = Field(ge=0, le=1800, default=300, description="Number of seconds to wait before a security policy match for an idle TCP session (0 - 1800, default = 300, 0 means unlimited).")    
    tcp_halfopen_match_timeout: int | None = Field(ge=0, le=300, default=8, description="Number of seconds to wait before a security policy match for a session after one peer has sent an open session packet but the other has not responded (0 - 300, default = 8, 0 means unlimited).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "NgfwSettingsModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "NgfwSettingsModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:12.547357Z
# ============================================================================


"""
Pydantic Models for CMDB - system/session_helper

Runtime validation models for system/session_helper configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional
from enum import Enum

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class SessionHelperNameEnum(str, Enum):
    """Allowed values for name field."""
    FTP = "ftp"
    TFTP = "tftp"
    RAS = "ras"
    H323 = "h323"
    TNS = "tns"
    MMS = "mms"
    SIP = "sip"
    PPTP = "pptp"
    RTSP = "rtsp"
    DNS_UDP = "dns-udp"
    DNS_TCP = "dns-tcp"
    PMAP = "pmap"
    RSH = "rsh"
    DCERPC = "dcerpc"
    MGCP = "mgcp"


# ============================================================================
# Main Model
# ============================================================================

class SessionHelperModel(BaseModel):
    """
    Pydantic model for system/session_helper configuration.
    
    Configure session helper.
    
    Validation Rules:        - id_: min=0 max=4294967295 pattern=        - name: pattern=        - protocol: min=0 max=255 pattern=        - port: min=1 max=65535 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    id_: int | None = Field(ge=0, le=4294967295, default=0, serialization_alias="id", description="Session helper ID.")    
    name: SessionHelperNameEnum = Field(description="Helper name.")    
    protocol: int = Field(ge=0, le=255, default=0, description="Protocol number.")    
    port: int = Field(ge=1, le=65535, default=0, description="Protocol port.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SessionHelperModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SessionHelperModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:14.049770Z
# ============================================================================


"""Type stubs for FORTIGUARD category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .answers import Answers
    from .redirect_portal import RedirectPortal
    from .service_communication_stats import ServiceCommunicationStats

__all__ = [
    "Answers",
    "RedirectPortal",
    "ServiceCommunicationStats",
    "Fortiguard",
]


class Fortiguard:
    """FORTIGUARD API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    answers: Answers
    redirect_portal: RedirectPortal
    service_communication_stats: ServiceCommunicationStats

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize fortiguard category with HTTP client."""
        ...



"""FortiOS CMDB - Firewall category (stub)"""

from typing import Any
from ..firewall_base import Firewall as FirewallBase
from .auth import Auth
from .count import Count
from .deauth import Deauth

class Firewall(FirewallBase):
    """Firewall endpoints wrapper for CMDB API."""

    auth: Auth
    count: Count
    deauth: Deauth

    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
